@@ -113,7 +113,7 @@
 
 .producePanelFirstSn <- function(panelFirst,
                                 x.ticks, scaleX, scaleX.fct,
-                                y.ticks, scaleY, scaleY.fct){
+                                y.ticks, scaleY, scaleY.fct, logArg){
 
 
   if(is.null(scaleX.fct)) scaleX.fct <- pnorm
@@ -122,13 +122,15 @@
   if(!is.null(x.ticks)){
      .xticksS <- substitute({.x.ticks <- x0}, list(x0 = if(scaleX) x.ticks else scaleX.fct(x.ticks)))
   }else{
+     logx <- FALSE
+     if(!is.null(logArg)) if(grepl("x",logArg)) logx <- TRUE
      if(!scaleX){
-        .xticksS <- substitute({
-               .x.ticks <- axTicks(1, axp=par("xaxp"), usr=par("usr"))
-               })
+        .xticksS <- if(logx)
+                    substitute(.x.ticks <- axTicks(1, log=TRUE)) else
+                    substitute(.x.ticks <- axTicks(1, axp=par("xaxp"), usr=par("usr")))
      }else{
         .xticksS <- substitute({
-               .x.ticks <- fct(axTicks(1, axp=par("xaxp"), usr=par("usr")))
+                   .x.ticks <- fct(axTicks(1, axp=par("xaxp"), usr=par("usr")))
                 },list(fct=scaleX.fct))
     }
   }
@@ -136,10 +138,12 @@
   if(!is.null(y.ticks)){
      .yticksS <- substitute({.y.ticks <- y0}, list(y0 = if(scaleY) y.ticks else scaleY.fct(y.ticks)))
   }else{
+     logy <- FALSE
+     if(!is.null(logArg)) if(grepl("y",logArg)) logy <- TRUE
      if(!scaleY){
-        .yticksS <- substitute({
-               .y.ticks <- axTicks(2, axp=par("yaxp"), usr=par("usr"))
-               })
+        .yticksS <- if(logy)
+                    substitute(.y.ticks <- axTicks(2, log=TRUE)) else
+                    substitute(.y.ticks <- axTicks(2, axp=par("yaxp"), usr=par("usr")))
      }else{
         .yticksS <- substitute({
                .y.ticks <- fct(axTicks(2, axp=par("yaxp"), usr=par("usr")))
@@ -308,7 +312,7 @@
                 #stop("Argument 'inner' must either be 'logical' or a 'list'")
                 if(!is.list(inner))
                     inner <- as.list(inner)
-                inner <- lapply(.mpresubs,inner)
+                inner <- lapply(inner,.mpresubs)
                 innerT <- .fillList(inner,(type=="info")+dims)
                 inf1 <- 0
                 if(type=="info"){ if(1 %in% to.draw) inf1 <- 1}
@@ -377,15 +381,15 @@
                         scaleX[i] <- scaleX[i] & !grepl("x",logArg[i])
                         scaleY[i] <- scaleY[i] & !grepl("y",logArg[i])
                      }
-                     if(with.automatic.grid[i]&&
-                        (scaleX[i]||scaleY[i])
+                     if(with.automatic.grid[i] # &&
+                       # (scaleX[i]||scaleY[i])
                      ){
                         pF[[i]] <-  .producePanelFirstSn(
                              pF.L[[i]],
                               x.ticks = x.ticks[[i]], scaleX = scaleX[i],
                                   scaleX.fct = scaleX.fct[[i]],
                               y.ticks = y.ticks[[i]], scaleY = scaleY[i],
-                                  scaleY.fct = scaleY.fct[[i]])
+                                  scaleY.fct = scaleY.fct[[i]], logArg[i])
                      }
             }
 
